import React, { useEffect, useState } from "react";
import {
  Button,
  TextField,
  Dialog,
  DialogActions,
  DialogContent,
  DialogTitle,
  Typography,
  Tooltip,
  CircularProgress,
  Autocomplete,
  Alert
} from "@mui/material";
import { useCSEntrySearch, useTagsAndCommitIds } from "../../../api/SwaggerApi";
import { useTypingTimer } from "../../common/SearchBoxFilter/TypingTimer";
import { formatDate, transformHostQuery } from "../../common/Helper";

export function IOCDeployDialog({
  open,
  setOpen,
  iocId,
  submitCallback,
  hasActiveDeployment,
  init = {},
  error,
  resetError
}) {
  function onError(message) {
    console.error(message);
  }

  const [hosts, getHosts, , loadingHosts] = useCSEntrySearch();
  const [host, setHost] = useState(init);
  const [query, onHostKeyUp] = useTypingTimer({ interval: 500 });
  const [tagOrCommitId, getTagOrCommitId, , loadingTagsAndCommitIds] =
    useTagsAndCommitIds(onError);
  const [gitRepo, setGitRepo] = useState(init.git || "");
  const [gitVersion, setGitVersion] = useState(init.version || "");
  const [queryGitVersion, onGitVersionKeyUp] = useTypingTimer({
    interval: 500
  });
  const gitProjectId = init.gitProjectId;

  const handleClose = () => {
    setOpen(false);
  };

  useEffect(
    () => getHosts({ query: transformHostQuery(`${query}`) }),
    [query, getHosts]
  );
  useEffect(() => {
    getTagOrCommitId(gitProjectId, queryGitVersion, false, "CONTAINS");
  }, [queryGitVersion, gitProjectId, getTagOrCommitId]);

  const onSubmit = (event) => {
    event.preventDefault();
    const { git: gitText } = event.currentTarget.elements;
    const git = gitText.value;

    submitCallback(
      {
        ioc_id: iocId
      },
      {
        requestBody: {
          sourceUrl: git,
          sourceVersion: gitVersion,
          hostCSEntryId: host
            ? Number(host.csEntryHost.id)
            : init.csEntryHost
            ? init.csEntryHost.id
            : undefined
        }
      }
    );
  };

  // Creates, and formats tags/commitIds with dates into a table-format for using it in autocomplete
  function createOptionTags(option) {
    return (
      <table>
        <tr>
          <td style={{ width: "7em", maxWidth: "7em" }}>
            <Typography
              fontFamily="monospace"
              fontWeight="bolder"
              overflow="hidden"
              textOverflow="ellipsis"
              whiteSpace="nowrap"
            >
              {option.shortReference}
            </Typography>
          </td>
          <td style={{ width: "1em" }}>-</td>
          <td>
            <Typography
              fontFamily="monospace"
              display="inline-block"
            >
              {formatDate(option.commitDate)}
            </Typography>
          </td>
        </tr>
      </table>
    );
  }

  return (
    <Dialog
      open={open}
      onClose={handleClose}
    >
      <form onSubmit={onSubmit}>
        <DialogTitle
          id="form-dialog-title"
          variant="h2"
        >
          {hasActiveDeployment ? "Deploy revision" : "Deploy"}
        </DialogTitle>
        <DialogContent
          sx={{
            gap: 1.5,
            display: "flex",
            flexDirection: "column",
            minWidth: "600px"
          }}
        >
          <TextField
            autoComplete="off"
            id="git"
            label="Git repository"
            variant="standard"
            defaultValue={init.git || ""}
            fullWidth
            onChange={(event) => {
              setGitRepo(event.target.value);
              resetError();
            }}
            disabled
            required
          />

          <Autocomplete
            autoHighlight
            id="version"
            options={tagOrCommitId}
            loading={loadingTagsAndCommitIds}
            clearOnBlur={false}
            defaultValue={
              init.version
                ? {
                    reference: init.version,
                    shortReference: init.shortVersion
                      ? init.shortVersion
                      : init.version
                  }
                : null
            }
            getOptionLabel={(option) => {
              return option.shortReference;
            }}
            renderOption={(props, option) => {
              return (
                <li {...props}>
                  <Tooltip title={option.description}>
                    <div>{createOptionTags(option)}</div>
                  </Tooltip>
                </li>
              );
            }}
            onChange={(event, value, reason) => {
              setGitVersion(value?.reference);
              resetError();
              getTagOrCommitId(
                gitProjectId,
                value?.reference,
                false,
                "CONTAINS"
              );
            }}
            onInputChange={(event, value, reason) => {
              event && onGitVersionKeyUp(event.nativeEvent);
            }}
            disabled={!gitRepo || gitRepo.trim() === ""}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Revision"
                variant="outlined"
                fullWidth
                required
                InputProps={{
                  ...params.InputProps,
                  endAdornment: (
                    <React.Fragment>
                      {loadingTagsAndCommitIds ? (
                        <CircularProgress
                          color="inherit"
                          size={20}
                        />
                      ) : null}
                      {params.InputProps.endAdornment}
                    </React.Fragment>
                  )
                }}
              />
            )}
            autoSelect
          />

          <Autocomplete
            autoHighlight
            id="host"
            options={hosts.hostList}
            loading={loadingHosts}
            clearOnBlur={false}
            defaultValue={init}
            getOptionLabel={(option) => {
              return option?.csEntryHost?.fqdn;
            }}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Host"
                variant="outlined"
                required
                InputProps={{
                  ...params.InputProps,
                  endAdornment: (
                    <React.Fragment>
                      {loadingHosts ? (
                        <CircularProgress
                          color="inherit"
                          size={20}
                        />
                      ) : null}
                      {params.InputProps.endAdornment}
                    </React.Fragment>
                  )
                }}
              />
            )}
            onChange={(event, value, reason) => {
              setHost(value);
              resetError();
            }}
            onInputChange={(event, value, reason) => {
              event && onHostKeyUp(event.nativeEvent);
            }}
            disabled={hasActiveDeployment}
            autoSelect
            filterOptions={(options, state) => options}
          />

          {hasActiveDeployment ? (
            <Typography variant="body2">
              Hint: First undeploy IOC if you want to move to another host.
            </Typography>
          ) : (
            <></>
          )}

          {error ? <Alert severity="error">{error}</Alert> : <></>}
        </DialogContent>
        <DialogActions>
          <Button
            onClick={handleClose}
            color="primary"
          >
            Cancel
          </Button>
          <Button
            color="primary"
            variant="contained"
            type="submit"
            disabled={!host || !gitVersion}
          >
            Deploy
          </Button>
        </DialogActions>
      </form>
    </Dialog>
  );
}
